/*
*	Cd data reading sample, uses a caching system to remove the
*	need to search for any files after the initialization.
*
*       cd.c and cd.h contain the data files required, replace these
*       with your own 
*
*	CHANGED		PROGRAMMER		REASON
*	-------  	----------  	------
*	02/12/97	Mike Kav		Created
*
*/

#include <sys/types.h>
#include <kernel.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libgs.h>
#include <libcd.h>
#include "main.h"

// For handling pad
#include "pad.h"
// loading functions
#include "cd.h"

// Function Prototypes
void main(void);
void LoadTestFiles(void);
void Demo(void);
void CleanUp(void);
void ResetDemo(void);
void VRAM(void);
void displayscreen(u_long* addr);
void initGraph(void);

#define	SCREEN_X		320		
#define	SCREEN_Y		256


// Buffers for test program
unsigned char analogbuf[ 131092 ];
unsigned char backbuf[ 131092 ];
unsigned char warningbuf[ 163860 ];
unsigned char memsprtbuf[ 131092 ];
unsigned char sonybuf[ 163860 ];
unsigned char tilebuf[ 131092 ];

unsigned char testanalogbuf[ 131092 ];
unsigned char testbackbuf[ 131092 ];
unsigned char testwarningbuf[ 163860 ];
unsigned char testmemsprtbuf[ 131092 ];
unsigned char testsonybuf[ 163860 ];
unsigned char testtilebuf[ 131092 ];

int currentBuffer;

void main(void)
{
	int buttonValue=0;	// Value returned from pad

	PadInit(0);
	InitCd();

// Load up .TIM's for comparison later
	LoadTestFiles();

	if(!CacheFilePos())
	{
		printf("Failed To Find All Files\n");
		return;
	}

	initGraph();
        printf("Initialised Graphics System,press X\n");

	while((buttonValue = padButton()) != LEFT_SELECT)
	{
		switch(buttonValue)
		{
			case LEFT_CROSS:
			{	
					Demo();
					break;
			}
			case LEFT_CIRCLE:
			{
					displayFilePositions();
					break;
			}
			case LEFT_SQUARE:
			{
				VRAM();
				buttonValue = 0;
				break;
			}
		}

	}

	CleanUp();
	ResetGraph(1);
	StopCallback();
    return;
}

/*
*
*	NAME		void Demo(void)
*
*	FUNCTION	loads files from CD
*
*	NOTES		
*
*
*	CHANGED		PROGRAMMER		REASON
*	-------  	----------  	------
*	09/12/97	Mike Kav		Created
*
*/
void Demo(void)
{
	int i;
	int startTime;
	int readbytes;

	printf("Enter Demo\n");

	printf("\n** Loading Piracy Screen\n");
	startTime=VSync(-1);
	readbytes = CdLoadFile(WARNING,(unsigned long*)testwarningbuf);
	printf("\n***** Load Time:%d,bytes read:%d *****",VSync(-1)-startTime,readbytes);
	printf( "\nCompare %d",memcmp( testwarningbuf,warningbuf,163860 ) );
	printf( ", Original Data at %X",testwarningbuf );
	printf( ", Reference data at %X\n",warningbuf );
	displayscreen(testwarningbuf);

	printf("\n** Loading Sony Screen\n");
	startTime=VSync(-1);
	readbytes = CdLoadFile(SONY,(unsigned long*)testsonybuf);
	printf("\n***** Load Time:%d,bytes read:%d *****",VSync(-1)-startTime,readbytes);
	printf( "\nCompare %d",memcmp( testsonybuf,sonybuf,10 ) );
	printf( ", Original Data at %X",testsonybuf );
	printf( ", Reference data at %X\n",sonybuf );
	displayscreen((unsigned long*)testsonybuf);
									
	printf("\n** Loading Analog Screen\n");
	startTime=VSync(-1);
	readbytes = CdLoadFile(ANALOG,(unsigned long*)testanalogbuf);
	printf("\n***** Load Time:%d,bytes read:%d *****",VSync(-1)-startTime,readbytes);
	printf( "\nCompare %d",memcmp( testanalogbuf,analogbuf,131092 ) );
	printf( ", Original Data at %X",testanalogbuf );
	printf( ", Reference data at %X\n",analogbuf );
	displayscreen((unsigned long*)testanalogbuf);

	printf("\n** Loading Backdrop Screen\n");
	startTime=VSync(-1);
	readbytes = CdLoadFile(BACK,(unsigned long*)testbackbuf);
	printf("\n***** Load Time:%d,bytes read:%d *****",VSync(-1)-startTime,readbytes);
	printf( "\nCompare %d",memcmp( testbackbuf,backbuf,131092 ) );
	printf( ", Original Data at %X",testbackbuf );
	printf( ", Reference data at %X\n",backbuf );
	displayscreen(testbackbuf);

	printf("\n** Loading Tiles Screen\n");
	startTime=VSync(-1);
	readbytes = CdLoadFile(TILES,(unsigned long*)testtilebuf);
	printf("\n***** Load Time:%d,bytes read:%d *****",VSync(-1)-startTime,readbytes);
	printf( "\nCompare %d",memcmp( testtilebuf,tilebuf,131092 ) );
	printf( ", Original Data at %X",testtilebuf );
	printf( ", Reference data at %X\n",tilebuf );
	displayscreen(testtilebuf);

	printf("\n** Loading Memory Card Screen\n");
	startTime=VSync(-1);
	readbytes = CdLoadFile(MEMORY,(unsigned long*)testmemsprtbuf);
	printf("\n***** Load Time:%d,bytes read:%d *****",VSync(-1)-startTime,readbytes);
	printf( "\nCompare %d",memcmp( testmemsprtbuf,memsprtbuf,131092 ) );
	printf( ", Original Data at %X",testmemsprtbuf );
	printf( ", Reference data at %X\n",memsprtbuf );
	displayscreen(testmemsprtbuf);

	return;
}

/*
*
*	NAME		void CleanUp(void)
*
*	FUNCTION	Removes CD system etc ..
*
*	NOTES		
*
*
*	CHANGED		PROGRAMMER		REASON
*	-------  	----------  	------
*	09/12/97	Mike Kav		Created
*
*/
void CleanUp(void)
{
	// Clear up cd system
	DeInitCd();
	StopCallback();      /* Stop the CD callbacks. */
	ResetGraph(3);       /* GPU warm reset */
	
	return;
}

/*
*
*	NAME		void initGraph(void)
*
*	FUNCTION	Uses Gs to initialize graphics
*
*	NOTES		
*
*
*	CHANGED		PROGRAMMER		REASON
*	-------  	----------  	------
*	09/12/97	Mike Kav		Created
*
*/
void initGraph(void)
{

	FntLoad(960,256);
	FntOpen(-96,-96,192,192,0,512);

	SetVideoMode(MODE_PAL);
	GsInitGraph(SCREEN_X, SCREEN_Y,GsINTER|GsOFSGPU,1,0);
	GsDefDispBuff(0,0,0,SCREEN_Y);
	SetBackColor(128, 128, 128);	

}


/*
*
*	NAME		void displayscreen(u_long *addr)
*
*	FUNCTION	Displays 16-bit .TIM on screen
*
*	NOTES		
*
*
*	CHANGED		PROGRAMMER		REASON
*	-------  	----------  	------
*	09/12/97	Mike Kav		Created
*
*/
void displayscreen(u_long *addr)
{
	// 5 seconds worth
//	int timer=5*60;
	int timer=1*60;

	GsIMAGE theImage;
	RECT r={0,0,320,256};	

	// Get TIM image details

	GsGetTimInfo(++addr,&theImage);
	printf("Image Data-Pixel Mode:%d,width:%d,height:%d\n",theImage.pmode,theImage.pw,theImage.ph);
	
	r.w = theImage.pw;
	r.h = theImage.ph;

	SetDispMask(1);

		// Quick way of displaying 16 bit TIM on screen
	while(timer>0)
	{
		currentBuffer=GsGetActiveBuff();
		LoadImage(&r,theImage.pixel);		
		DrawSync(0);
		VSync(0);
		timer--;
	}	  

	return;
}

/*
*
*	NAME		void VRAM(void)
*
*	FUNCTION	Have a look round VRAM
*
*	NOTES		
*
*
*	CHANGED		PROGRAMMER		REASON
*	-------  	----------  	------
*	09/12/97	Mike Kav		Created
*
*/
void VRAM(void)
{
	RECT	vram;
	DISPENV	disp;
	u_long	padd;

	while (((padd = PadRead(1))&PADk) == 0) {
		
		if (padd & PADl)	disp.screen.w -= 2;
		if (padd & PADm)	disp.screen.w += 2;
		if (padd & PADn)	disp.screen.h -= 2;
		if (padd & PADo)	disp.screen.h += 2;
		
		if (padd & PADLleft)	disp.disp.x -= 2;
		if (padd & PADLright)	disp.disp.x += 2;
		if (padd & PADLup)	disp.disp.y -= 2;
		if (padd & PADLdown)	disp.disp.y += 2;
		
		if (padd & PADRleft)	disp.screen.x -= 2;
		if (padd & PADRright)	disp.screen.x += 2;
		if (padd & PADRup)	disp.screen.y -= 2;
		if (padd & PADRdown)	disp.screen.y += 2;

		/* Clear VRAM if start pressed. */
		if (padd & PADstart) {
		    setRECT(&vram, 0, 0, 1023, 511);
		    ClearImage(&vram, 0, 0, 40);
		}
		
		VSync(0);
		GsSwapDispBuff();
	}
}

/*
*
*	NAME		void LoadTestFiles(void)
*
*	FUNCTION	Loads files to compare against my loading code
*
*	NOTES		
*
*
*	CHANGED		PROGRAMMER		REASON
*	-------  	----------  	------
*	09/12/97	Mike Kav		Created
*
*/
void LoadTestFiles(void)
{

	int temp;
	int readbytes;

	temp=VSync(-1);
	readbytes = CdReadFile( "\\TIM\\BACK.TIM;1",(u_long *)backbuf,0 );
	CdReadSync( 0,0 );
	printf("***** Load Time:%d,bytes read:%d *****\n",VSync(-1)-temp,readbytes);

	temp=VSync(-1);
	readbytes = CdReadFile( "\\TIM\\ANALOG.TIM;1",(u_long *)analogbuf,0 );
	CdReadSync( 0,0 );
	printf("***** Load Time:%d,bytes read:%d *****\n",VSync(-1)-temp,readbytes);

	temp=VSync(-1);
	readbytes = CdReadFile( "\\TIM\\DWARNING.TIM;1",(u_long *)warningbuf,0 );
	CdReadSync( 0,0 );
	printf("***** Load Time:%d,bytes read:%d *****\n",VSync(-1)-temp,readbytes);

	temp=VSync(-1);
	readbytes = CdReadFile( "\\TIM\\MEMSPRT.TIM;1",(u_long *)memsprtbuf,0 );
	CdReadSync( 0,0 );
	printf("***** Load Time:%d,bytes read:%d *****\n",VSync(-1)-temp,readbytes);

	temp=VSync(-1);
	readbytes = CdReadFile( "\\TIM\\TILES.TIM;1",(u_long *)tilebuf,0 );
	CdReadSync( 0,0 );
	printf("***** Load Time:%d,bytes read:%d *****\n",VSync(-1)-temp,readbytes);

	temp=VSync(-1);
	readbytes = CdReadFile( "\\TIM\\SONY.TIM;1",(u_long *)sonybuf,0 );
	CdReadSync( 0,0 );
	printf("***** Load Time:%d,bytes read:%d *****\n",VSync(-1)-temp,readbytes);

	return;

}
