/* $PSLibId: Runtime Library Release 3.6$ */
/*			    tuto0
 *			
 :	movie mapping on the surface of the objects
 *	   IuWFNg̕\ʂɃ}bv
 *
 *		Copyright (C) 1994 by Sony Corporation
 *			All rights Reserved
 *
 *	 Version	Date		Design
 *	----------------------------------------------------	
 *	1.00		Jul,14,1994	yutaka
 *	1.10		Sep,01,1994	suzu
 *	2.00		Feb,8,1996	suzu	(rewrite)
 */

#include <sys/types.h>	
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>

#define FILE_NAME   "\\DATA\\MOV.STR;1"

/*
 * Triangle primitive buffer. This buffer should be allocaed dynamically
 * according to total primitive size by malloc()
 : Rp`|SpPbg
 * _EpPbgobt@́Amalloc() œIɊtׂłB	
 */
#define MAX_POLY	512			/* max polygon (per object) */

/* object definition : IuWFNg̒` */
typedef struct {
	MATRIX		m;	/* local-local matrix: [J}gNX */
	MATRIX		lw;	/* local-world matrix: [J}gNX */
	SVECTOR		v;	/* local vector: [JxNg */
	int		n;	/* number of polygons: |S */
	struct {		/* polygon information: |S̏ */
		POLY_FT3	prim[2];
		SVECTOR		v0, v1, v2;
	} p[MAX_POLY];
} OBJ_FT3;

#define SCR_Z		1024		/* projection: eʂ܂ł̋ */
#define OTLENGTH	12		/* depth of OT: ns̒i */
#define OTSIZE		(1<<OTLENGTH)	/* depth of OT: ns̒i */
#define MAX_OBJ		16		/* max object: IuWFNǧ */

#define MODELADDR	((u_long *)0x80120000)	/* TMD address */
#define TEXADDR		((u_long *)0x80140000)	/* TIM address */

static int pad_read(OBJ_FT3 *obj, int nobj, MATRIX *m);
static void set_position(OBJ_FT3 *obj, int n);
static void add_OBJ_FT3(MATRIX *ws, u_long *ot, OBJ_FT3 *obj, int id);
static void loadTIM(u_long *addr);
static void setSTP(u_long *col, int n);
static int loadTMD_FT3(u_long *tmd, OBJ_FT3 *obj);

main()
{
	static SVECTOR	ang = {0,0,0};	/* self-rotation angle; ]p */
	MATRIX	rot;			/* self-rotation angle: ]p */
	MATRIX	ws;			/* world matrix:[h}gNX */
	OBJ_FT3	obj[MAX_OBJ];		/* object:IuWFNg */
	u_long	ot[2][OTSIZE];		/* OT: ns obt@*/
	int	nobj = 1;		/* number of obj: IuWFNǧ */
	int	id   = 0;		/* buffer ID: pPbg ID */
	int	i, n; 			/* work */
	RECT	rect;			/* work */
	
	/* init CD */
	CdInit();
	
	/* initialize frame double buffer: _uobt@̏ */
	db_init(640, /*480*/240, SCR_Z, 60, 120, 120);	

	/* load TIM to frame buffer: TIM t[obt@ɓ] */
	loadTIM(TEXADDR);	

	/* read each TMD: TMD ΂΂ɂēǂݍ */
	for (i = 0; i < MAX_OBJ; i++) 
		loadTMD_FT3(MODELADDR, &obj[i]);	
	
	/* layout each object: IuWFNgʒuCAEg */
	set_position(obj, 0);		
	
	/* clear texture buffer */
	setRECT(&rect, 640, 0, 256, 256);
	ClearImage(&rect, 64, 64, 64);

	/* start animation */
	animInit(FILE_NAME, 640, 32);
	
	/* start display;\Jn */
	SetDispMask(1);			
	
	/* main loop: C[v */
	while ((nobj = pad_read(obj, nobj, &ws)) != -1) {
		
		/* pollling animation */
		animPoll();
		
		/* swap primitive buffer ID:pPbgobt@ IDXbv */
		id = id? 0: 1;
		
		/* clear OT:nsNA */
		ClearOTagR(ot[id], OTSIZE);			

		/* rotate matrix of the earth: n] */
		ang.vy += 32;
		RotMatrix(&ang, &rot);
		
		/* set primitives: v~eBu̐ݒ */
		for (i = 0; i < nobj; i++) {
			MulMatrix0(&obj[i].m, &rot, &obj[i].lw);
			add_OBJ_FT3(&ws, ot[id], &obj[i], id);
		}
		
		/* print debug information: fobOXgO̐ݒ */
		FntPrint("polygon=%d\n", obj[0].n);
		FntPrint("objects=%d\n", nobj);
		FntPrint("total  =%d\n", obj[0].n*nobj);
		
		/* draw OT and swap frame double buffer
		 : _uobt@̃XbvƂns`
		 */
		db_swap(&ot[id][OTSIZE-1]);
	}
	PadStop();
	StopCallback();
}

/*
 * Read controler and set world-screen matrix
 : pbh̓ǂݍ
 * ]̒SʒuƉ]pxǂݍ݃[hXN[}gNXݒ
 * 	
 */	
static int pad_read(OBJ_FT3 *obj, int nobj, MATRIX *m)
{
	static SVECTOR	ang   = {0, 0, 0};
	static VECTOR	vec   = {0, 0, SCR_Z*3};
	static int	scale = ONE;
	static int	opadd = 0;
	
	VECTOR	svec;
	int 	padd = PadRead(1);
	
	if (padd & PADk)	return(-1);
	if (padd & PADRup)	ang.vx += 8;
	if (padd & PADRdown)	ang.vx -= 8;
	if (padd & PADRleft) 	ang.vy += 8;
	if (padd & PADRright)	ang.vy -= 8;
	if (padd & PADL1)	vec.vz += 8;
	if (padd & PADL2) 	vec.vz -= 8;
	
	if ((opadd==0) && (padd & PADLup))	set_position(obj, nobj++);
	if ((opadd==0) && (padd & PADLdown))	nobj--;
	
	limitRange(nobj, 1, MAX_OBJ);
	opadd = padd;
	
	setVector(&svec, scale, scale, scale);
	RotMatrix(&ang, m);	
	
#ifdef ROTATE_YORSELF	/* rotate your camera: ώi) */
	{
		VECTOR	vec2;
		ApplyMatrixLV(m, &vec, &vec2);
		TransMatrix(m, &vec2);
		dumpVector("vec2=", &vec2);
	}
#else			/* rotate the world: qώiE) */
	TransMatrix(m, &vec);	
#endif
	/* set world screen matrix with aspecto ratio correction
	 : [hXN[}gNXAXyNg␳ɐݒ
	 */
	db_set_matrix(m);
	/*
	SetRotMatrix(m);
	SetTransMatrix(m);
	*/
	return(nobj);
}


/*
 * layout many object in the world-coordinate.
 * Since the position of each object is determined by rand(),
 * two objects may be located at the same position.
 : IuWFNg[hWnɃCAEg
 * ł͗gpēKɔzuĂ	
 * ̂߁A̃IuWFNgʒuɂԂ邱ƂB	
 */	
#define UNIT	400		/* resolution: ŏ𑜓x */

static void set_position(OBJ_FT3 *obj, int n)
{
	SVECTOR	ang;

	static loc_tab[][3] = {
		 0, 0, 0,
		 1, 0, 0,	0, 1, 0,	 0, 0, 1,
		-1, 0, 0,	0,-1, 0,	 0, 0,-1,
		 1, 1, 0,	0, 1, 1,	 1, 0, 1,
		-1,-1, 0,	0,-1,-1,	-1, 0,-1,
		 1,-1, 0,	0,-1, 1,	-1, 0, 1,
		-1, 1, 0,	0, 1,-1,	 1, 0,-1,
	};
	
	/* set axis of each earth: en̒nݒ */
	ang.vx = rand()%4096;
	ang.vy = rand()%4096;
	ang.vz = rand()%4096;
	RotMatrix(&ang, &obj[n].m);	
	
	/* set position of each earth: en̏ꏊݒ */
	obj[n].lw.t[0] = loc_tab[n][0]*UNIT;
	obj[n].lw.t[1] = loc_tab[n][1]*UNIT;
	obj[n].lw.t[2] = loc_tab[n][2]*UNIT;
}

/*
 * append object to OT: IuWFNg̓o^
 */
static void add_OBJ_FT3(MATRIX *ws, u_long *ot, OBJ_FT3 *obj, int id)
{
	MATRIX	ls;		/* local-screen matrix */
	
	/* push current matrix: Jg}gNXޔ */
	PushMatrix();				

	/* make local-screen coordinate
	 :  [JXN[}gNX
	 */
	CompMatrix(ws, &obj->lw, &ls);

	/* set matrix: [JXN[}gNXݒ */
	SetRotMatrix(&ls);		/* set matrix */
	SetTransMatrix(&ls);		/* set vector */
	
	/* rotate-translate-perspective translation:]EړEϊ */
	RotPMD_FT3((long *)&obj->n, ot, OTLENGTH, id, 0);

	/* recover old matrix: }gNXɂǂă^[ */
	PopMatrix();
}

/*
 * Load TIM data from main memory to the frame buffer
 : TIM [h
 */	
static void loadTIM(u_long *addr)
{
	TIM_IMAGE	image;		/* TIM header */
	
	OpenTIM(addr);			/* open TIM */
	while (ReadTIM(&image)) {
		if (image.caddr) {	/* load CLUT (if needed) */
			setSTP(image.caddr, image.crect->w);
			LoadImage(image.crect, image.caddr);
		}
		if (image.paddr) 	/* load texture pattern */
			LoadImage(image.prect, image.paddr);
	}
}
	
/*
 * change STP on texture pattern to inhibit transparent color
 : F֎~邽߂ STP bit  1 ɂ
 */	
static void setSTP(u_long *col, int n)
{
	n /= 2;  
	while (n--) 
		*col++ |= 0x80008000;
}

/*
 * parse TMD: TMD ̉
 */
static int loadTMD_FT3(u_long *tmd, OBJ_FT3 *obj)
{
	TMD_PRIM	tmdprim;
	int		col, i, n_prim = 0;

	/* open TMD: TMD ̃I[v */
	if ((n_prim = OpenTMD(tmd, 0)) > MAX_POLY) 
		n_prim = MAX_POLY;
	
	/*
	 * Set unchanged member of primitive here to deliminate main
	 * memory write access
	 : CgANZX炷߂Ƀv~eBuobt@̂A
	 * Ȃ̂炩ߐݒ肵Ă
	 */	 
	for (i = 0; i < n_prim && ReadTMD(&tmdprim) != 0; i++) {

		/* initialize primitive: v~eBȕ */
		SetPolyFT3(&obj->p[i].prim[0]);

		/* copy normal and vertex: @ƒ_xNgRs[ */
		copyVector(&obj->p[i].v0, &tmdprim.x0);
		copyVector(&obj->p[i].v1, &tmdprim.x1);
		copyVector(&obj->p[i].v2, &tmdprim.x2);
		
		/* lighting: vZiŏ̈̂݁j*/
		col = (tmdprim.n0.vx+tmdprim.n0.vy)*128/ONE/2+128;
		setRGB0(&obj->p[i].prim[0], col, col, col);
		
		/* copy texture point because this point never changes
		 * by rotation
		 : eNX`W͕ςȂ̂łŃRs[Ă
		 */
		setUV3(&obj->p[i].prim[0], 
		       tmdprim.u0, tmdprim.v0,
		       tmdprim.u1, tmdprim.v1,
		       tmdprim.u2, tmdprim.v2);
		
		/* copy tpage and clut
		 : eNX`y[W^eNX` CLUT ID Rs[
		 */
		obj->p[i].prim[0].tpage = GetTPage(2, 0, 640, 0);
		/*
		obj->p[i].prim[0].tpage = tmdprim.tpage;
		obj->p[i].prim[0].clut  = tmdprim.clut;
		*/
		/* duplicate primitive for primitive double buffering
		 : _uobt@gp̂Ńv~eBu̕
		 * gĂ
		 */  
		memcpy(&obj->p[i].prim[1],
		       &obj->p[i].prim[0], sizeof(POLY_FT3));
		
	}
	return(obj->n = i);
}
