/*
 * $PSLibId: Runtime Library Release 3.6$
 */
/*
 *	rcube: PS-X Demonstration program
 *
 *	"main.c" Main routine
 *
 *		Version 3.02	Jan, 9, 1995
 *
 *		Copyright (C) 1993,1994,1995 by Sony Computer Entertainment
 *			All rights Reserved
 */

#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libgs.h>
#include "table.h"
#include "pos.h"

/* Texture information 
 * eNX`
 */
#define TIM_ADDR 0x80020000		/* Address where target TIM file is stored 
/* gpTIMt@C̊i[AhX */

#define TIM_HEADER 0x00000010

/* Modeling data information 
 * fOf[^
 */
#define TMD_ADDR 0x80010000		/* Address where target TMD file is stored 
/* gpTMDt@C̊i[AhX */

u_long *TmdBase;			/* Address of object in TMD
/* TMD̂AIuWFNg̃AhX */

int CurrentTmd; 			/* TMD no. in use /* gpTMDԍ */

/* Ordering table (OT)
 * I[_Oe[u (OT)
 */
#define OT_LENGTH  7			/* OT resolution (size) /* OT𑜓xi傫j */
GsOT WorldOT[2];			/* OT info (double buffer) /* OTi_uobt@j */
GsOT_TAG OTTags[2][1<<OT_LENGTH];	/* OT's tag area (double buffer) /* OT̃^Öi_uobt@j */

/* GPU packet generation area 
 * GPUpPbg̈
 */
#define PACKETMAX 1000			/* Maximum number of packets per frame
/* 1t[̍őpPbg */

PACKET GpuPacketArea[2][PACKETMAX*64];	/* Packet area (double buffer) 
/* pPbg̈i_uobt@j */

/*  Object (cube) variable
 *  IuWFNgiL[ujϐ
 */
#define NCUBE 44
#define OBJMAX NCUBE
int nobj;				/* Number of cubes /* L[u */
GsDOBJ2 object[OBJMAX];			/* 3D object variable /* 3DIuWFNgϐ */
GsCOORDINATE2 objcoord[OBJMAX];		/* Local coordinate variable 
/* [JWϐ */

SVECTOR Rot[OBJMAX];			/* Rotation angle /* ]p */
SVECTOR RotV[OBJMAX];			/* Rotation speed (angular velocity)
/* ]Xs[hipxj */

VECTOR Trns[OBJMAX];			/* Cube position (translatio distance)
/* L[uʒuisړʁj */

VECTOR TrnsV[OBJMAX];			/* Translation speed /* ړXs[h */

/*  Viewpoint (VIEW)
 *  _iVIEWj
 */
GsRVIEW2  View;			/* Viewpoint variable /* _ϐ */
int ViewAngleXZ;		/* Viewpoint height /* _̍ */
int ViewRadial;			/* Distance from viewpoint /* _̋ */
#define DISTANCE 600		/* Initial value of radial /* Radial̏l */

/*  Light source
 *  
 */
GsF_LIGHT pslt[3];			/* Light source information variable x3
/* ϐ~3 */

/*  Other
 *  ̑...
 */
int Bakuhatu;				/* Explosion processing flag /* tO */
u_long PadData;				/* Control pad information /* Rg[pbh̏ */
u_long oldpad;				/* Pad information from previous frame /* Pt[Õpbh */
GsFOGPARAM dq;				/* Parameter for depth cueing (FOG) /* fvXL[(tHO)pp[^ */
int dqf;				/* FOG ON/OFF /* tHOONǂ */
int back_r, back_g, back_b;		/* Background color /* obNOEhF */

#define FOG_R 160
#define FOG_G 170
#define FOG_B 180

/*  Function prototype declaration 
 *  ֐̃vg^Cv錾
 */
void drawCubes();
int moveCubes();
void initModelingData();
void allocCube();
void initSystem();
void initAll();
void initTexture();
void initView();
void initLight();
void changeFog();
void changeTmd();

/*  Main routine
 *  C[`
 */
main()
{
	/* System initialization /* VXȅ */
	ResetCallback();
	initSystem();

	/* Other initialization /* ̑̏ */
	Bakuhatu = 0;
	PadData = 0;
	CurrentTmd = 0;
	dqf = 0;
	back_r = back_g = back_b = 0;
	initView();
	initLight(0, 0xc0);
	initModelingData(TMD_ADDR);
	initTexture(TIM_ADDR);
	allocCube(NCUBE);
	
	/* Main loop /* C[v */
	while(1) {
		if(moveCubes()==0)
		  return 0;
		GsSetRefView2(&View);
		drawCubes();
	}
}


/*  Draw 3D object (cube)
 *  3DIuWFNgiL[uj̕`
 */
void drawCubes()
{
	int i;
	GsDOBJ2 *objp;
	int activeBuff;
	MATRIX LsMtx;

	/* Which double buffer is active /* _uobt@̂ǂ炪ANeBuH */
	activeBuff = GsGetActiveBuff();

	/* Set GPU packet generation address to start of area
	/* GPUpPbgAhXGA̐擪ɐݒ */
	GsSetWorkBase((PACKET*)GpuPacketArea[activeBuff]);

	/* Clear contents of OT /* OT̓eNA */
	GsClearOt(0, 0, &WorldOT[activeBuff]);

	/* Register 3D object (cube to OT)
	/* 3DIuWFNgiL[ujOTւ̓o^ */
	objp = object;
	for(i = 0; i < nobj; i++) {

		/* Set Rotation angle to matrix /* ]p->}gNXɃZbg */
		RotMatrix(Rot+i, &(objp->coord2->coord));
		
		/* Reset flag after matrix has been updated
		/* }gNXXV̂ŃtOZbg */
		objp->coord2->flg = 0;

		/* Set Translation distance to matrix
		/* sړ->}gNXɃZbg */
		TransMatrix(&(objp->coord2->coord), &Trns[i]);
		
		/* Calculate matrix for perspective transformation and set to GTE
		/* ϊ̂߂̃}gNXvZĂfsdɃZbg */
		GsGetLs(objp->coord2, &LsMtx);
		GsSetLsMatrix(&LsMtx);
		GsSetLightMatrix(&LsMtx);

		/* Perform perspective transformation and register to OT
		/* ϊOTɓo^ */
		GsSortObject4(objp, &WorldOT[activeBuff], 14-OT_LENGTH,getScratchAddr(0));
		objp++;
	}

	/* Fetch pad contents for buffer /* pbh̓eobt@Ɏ荞 */
	oldpad = PadData;
	PadData = PadRead(1);
	
	/* Wait for V-BLANK /* V-BLANK҂ */
	VSync(0);
	
	/* Force termination of previous frame's drawing operation
	/* Õt[̕`ƂI */
	ResetGraph(1);

	/* Replace double buffer /* _uobt@ꊷ */
	GsSwapDispBuff();

	/* Insert screen clear command at start of OT
	/* OT̐擪ɉʃNA߂} */
	GsSortClear(back_r, back_g, back_b, &WorldOT[activeBuff]);

	/* Start drawing contents of OT as background
	/* OT̓eobNOEhŕ`Jn */
	GsDrawOt(&WorldOT[activeBuff]);
}

/* Cube translation
 *  L[üړ
 */
int moveCubes()
{
	int i;
	GsDOBJ2   *objp;
	
	/* exit program
	/* vOIăj^ɖ߂ */
	if((PadData & PADk)>0) {PadStop();ResetGraph(3);StopCallback();return 0;}
	
	/* Process according to pad value /* pbh̒lɂď */
	if((PadData & PADLleft)>0) {
		ViewAngleXZ++;
		if(ViewAngleXZ >= 72) {
			ViewAngleXZ = 0;
		}
	}
	if((PadData & PADLright)>0) {
		ViewAngleXZ--;
		if(ViewAngleXZ < 0) {
		  ViewAngleXZ = 71;
		}
	}
	if((PadData & PADLup)>0) View.vpy += 100;
	if((PadData & PADLdown)>0) View.vpy -= 100;
	if((PadData & PADRdown)>0) {
		ViewRadial-=3;
		if(ViewRadial < 8) {
			ViewRadial = 8;
		}
	}
	if((PadData & PADRright)>0) {
		ViewRadial+=3;
		if(ViewRadial > 450) {
			ViewRadial = 450;
		}
	}
	if((PadData & PADk)>0) return(-1);
	if(((PadData & PADRleft)>0)&&((oldpad&PADRleft) == 0)) changeFog();
	if(((PadData & PADRup)>0)&&((oldpad&PADRup) == 0)) changeTmd();
	if(((PadData & PADn)>0)&&((oldpad&PADn) == 0)) Bakuhatu = 1;
	if(((PadData & PADl)>0)&&((oldpad&PADl) == 0)) allocCube(NCUBE);

	View.vpx = rotx[ViewAngleXZ]*ViewRadial;
	View.vpz = roty[ViewAngleXZ]*ViewRadial;

	/* Update cube's position information /* L[üʒuXV */
	objp = object;
	for(i = 0; i < nobj; i++) {

		/* Start explosion /* ̊Jn */
		if(Bakuhatu == 1) {

			/* Auto-rotation speed /* ]x up */
			RotV[i].vx *= 3;
			RotV[i].vy *= 3;
			RotV[i].vz *= 3;

			/* Set translation direction and speed /* ړ&xݒ */
			TrnsV[i].vx = objp->coord2->coord.t[0]/4+
						(rand()-16384)/200;	
			TrnsV[i].vy = objp->coord2->coord.t[1]/6+
						(rand()-16384)/200-200;
			TrnsV[i].vz = objp->coord2->coord.t[2]/4+
						(rand()-16384)/200;
		}
		/* Processing during explosion /* ̏ */
		else if(Bakuhatu > 1) {
			if(Trns[i].vy > 3000) {
				Trns[i].vy = 3000-(Trns[i].vy-3000)/2;
				TrnsV[i].vy = -TrnsV[i].vy*6/10;
			}
			else {
				TrnsV[i].vy += 20*2;	/* Free fall /* R */
			}

			if((TrnsV[i].vy < 70)&&(TrnsV[i].vy > -70)&&
			   (Trns[i].vy > 2800)) {
				Trns[i].vy = 3000;
				TrnsV[i].vy = 0;

				RotV[i].vx *= 95/100;
				RotV[i].vy *= 95/100;
				RotV[i].vz *= 95/100;
			}


			TrnsV[i].vx = TrnsV[i].vx*97/100;
			TrnsV[i].vz = TrnsV[i].vz*97/100;
		}

		/* Update rotation angle /* ]p(Rotation)̍XV */
 		Rot[i].vx += RotV[i].vx;
		Rot[i].vy += RotV[i].vy;
		Rot[i].vz += RotV[i].vz;

		/* Upadte translation distance /* sړ(Transfer)̍XV */
 		Trns[i].vx += TrnsV[i].vx;
		Trns[i].vy += TrnsV[i].vy;
		Trns[i].vz += TrnsV[i].vz;

		objp++;
	}

	if(Bakuhatu == 1)
		Bakuhatu++;

	return(1);
}

/*	Map cube to initial position
 *  L[uʒuɔzu
 */
void allocCube(n)
int n;
{	
	int x, y, z;
	int i;
	int *posp;
	GsDOBJ2 *objp;
	GsCOORDINATE2 *coordp;

	posp = cube_def_pos;
	objp = object;
	coordp = objcoord;
	nobj = 0;
	for(i = 0; i < NCUBE; i++) {

		/* Initialize object structure /* IuWFNg\̂̏ */
		GsLinkObject4((u_long)TmdBase, objp, CurrentTmd);
		GsInitCoordinate2(WORLD, coordp);
		objp->coord2 = coordp;
		objp->attribute = 0;
		coordp++;
		objp++;

		/* Set initial position (read from pos.h)
		/* ʒu̐ݒ(pos.hǂ) */
		Trns[i].vx = *posp++;
		Trns[i].vy = *posp++;
		Trns[i].vz = *posp++;
		Rot[i].vx = 0;
		Rot[i].vy = 0;
		Rot[i].vz = 0;

		/* Initialize speed /* x̏ */
		TrnsV[i].vx = 0;
		TrnsV[i].vy = 0;
		TrnsV[i].vz = 0;
		RotV[i].vx = rand()/300;
		RotV[i].vy = rand()/300;
		RotV[i].vz = rand()/300;

		nobj++;
	}
	Bakuhatu = 0;
}

/*	Initialization functions
 *  CjVCY֐Q
 */
void initSystem()
{
	int i;

	/* Initialize pad /* pbh̏ */
	PadInit(0);

	/* Initialize graphic /* OtBbN̏ */
	GsInitGraph(640, 480, 2, 0, 0);
	GsDefDispBuff(0, 0, 0, 0);

	/* Initialize OT /* OT̏ */
	for(i = 0; i < 2; i++) {	
		WorldOT[i].length = OT_LENGTH;
		WorldOT[i].org = OTTags[i];
	}	

	/* Initialize 3D system /* 3DVXȅ */
	GsInit3D();
}

void initModelingData(tmdp)
u_long *tmdp;
{
	u_long size;
	int i;
	int tmdobjnum;
	
	/* Skip header /* wb_XLbv */
	tmdp++;

	/* Mapping to real address /* AhXփ}bsO */
	GsMapModelingData(tmdp);

	tmdp++;
	tmdobjnum = *tmdp;
	tmdp++; 		/* Point to top object /* 擪̃IuWFNg|Cg */
	TmdBase = tmdp;
}

/*	Read texture (transfer to VRAM)
 *  eNX`̓ǂݍ݁iVRAMւ̓]j
 */
void initTexture(tex_addr)
u_long *tex_addr;
{
	RECT rect1;
	GsIMAGE tim1;
	int i;
	
	while(1) {
		if(*tex_addr != TIM_HEADER) {
			break;
		}
		tex_addr++;	/* Skip header (1word) /* wb_̃XLbv(1word) */
		GsGetTimInfo(tex_addr, &tim1);
		tex_addr += tim1.pw*tim1.ph/2+3+1;	/* Go to next block 
			/* ̃ubN܂Ői߂ */

		rect1.x=tim1.px;
		rect1.y=tim1.py;
		rect1.w=tim1.pw;
		rect1.h=tim1.ph;
		LoadImage(&rect1,tim1.pixel);
		if((tim1.pmode>>3)&0x01) {	/* Transfer CLUT (if there are any) 
		/* CLUTΓ] */

			rect1.x=tim1.cx;
			rect1.y=tim1.cy;
			rect1.w=tim1.cw;
			rect1.h=tim1.ch;
			LoadImage(&rect1,tim1.clut);
			tex_addr += tim1.cw*tim1.ch/2+3;
		}
	}
}

/*	Initialize viewpoint
 *  _̏
 */
void initView()
{
	/* Set initial position as viewpoint variable
	/* ʒu_ϐɃZbg */
	ViewAngleXZ = 54;
	ViewRadial = 75;
	View.vpx = rotx[ViewAngleXZ]*ViewRadial;
	View.vpy = -100;
	View.vpz = roty[ViewAngleXZ]*ViewRadial;
	View.vrx = 0; View.vry = 0; View.vrz = 0;
	View.rz=0;

	/* Super coordinates of viewpoint /* _̐eW */
	View.super = WORLD;

	/* Setting /* ݒ */
	GsSetRefView2(&View);

	/* Projection */
	GsSetProjection(1000);

	/* Mode = 'normal lighting' */
	GsSetLightMode(0);
}

/*	Initialize light source 
 *  ̏
 */
void initLight(c_mode, factor)
int c_mode;	/* Light color = wight when 0, cocktail light when 1
/* ÔƂFAP̂ƂJNeCg */
int factor;	/* Intensity factor (0 to 255) /* 邳̃t@N^[(0`255) */
{
	if(c_mode == 0) {
		/* Set White light color /* F̃Zbg */
		pslt[0].vx = 200; pslt[0].vy= 200; pslt[0].vz= 300;
		pslt[0].r = factor; pslt[0].g = factor; pslt[0].b = factor;
		GsSetFlatLight(0,&pslt[0]);
		
		pslt[1].vx = -50; pslt[1].vy= -1000; pslt[1].vz= 0;
		pslt[1].r=0x20; pslt[1].g=0x20; pslt[1].b=0x20;
		GsSetFlatLight(1,&pslt[1]);
		
		pslt[2].vx = -20; pslt[2].vy= 20; pslt[2].vz= 100;
		pslt[2].r=0x0; pslt[2].g=0x0; pslt[2].b=0x0;
		GsSetFlatLight(2,&pslt[2]);
	}
	else {
		/* Cocktail light (used for Gouraud) 
		/* JNeCgiGouraudŎgpj */
		pslt[0].vx = 200; pslt[0].vy= 100; pslt[0].vz= 0;
		pslt[0].r = factor; pslt[0].g = 0; pslt[0].b = 0;
		GsSetFlatLight(0,&pslt[0]);
		
		pslt[1].vx = -200; pslt[1].vy= 100; pslt[1].vz= 0;
		pslt[1].r=0; pslt[1].g=0; pslt[1].b=factor;
		GsSetFlatLight(1,&pslt[1]);
		
		pslt[2].vx = 0; pslt[2].vy= -200; pslt[2].vz= 0;
		pslt[2].r=0; pslt[2].g=factor; pslt[2].b=0;
		GsSetFlatLight(2,&pslt[2]);
	}	

	/* Ambient (peripheral) light /* ArGgiӌj */
	GsSetAmbient(ONE/2,ONE/2,ONE/2);
}

/* FOG on/off
 * tHOON/OFF
 */
void changeFog()
{
	if(dqf) {
		/* Reset FOG /* tHÕZbg */
		GsSetLightMode(0);
		dqf = 0;
		back_r = 0;
		back_g = 0;
		back_b = 0;
	}
	else {
		/* Set Fog /* tHO̐ݒ */
		dq.dqa = -600;
		dq.dqb = 5120*4096;
		dq.rfc = FOG_R;
		dq.gfc = FOG_G;
		dq.bfc = FOG_B;
		GsSetFogParam(&dq);
		GsSetLightMode(1);
		dqf = 1;
		back_r = FOG_R;
		back_g = FOG_G;
		back_b = FOG_B;
	}
}

/*	Switch TMD data
 *  TMDf[^̐؂ւ
 */
void changeTmd()
{
	u_long *tmdp;
	GsDOBJ2 *objp;
	int i;

	/* Switch TMD /* TMD؂ւ */
	CurrentTmd++;
	if(CurrentTmd == 4) {
		CurrentTmd = 0;
	}
	objp = object;
	for(i = 0; i < nobj; i++) {
		GsLinkObject4((u_long)TmdBase, objp, CurrentTmd);
		objp++;
	}

	/* Switch light source color/intensity according to type of TMD
	/* TMD̎ނɂ킹Č̐F/邳؂ւ */
	switch(CurrentTmd) {
	    case 0:
                /* Normal (flat) /* m[} (flat) */
		initLight(0, 0xc0);
		break;
	    case 1:
	        /* Opaque (flat) /*  (flat) */
		initLight(0, 0xc0);
		break;
	    case 2:
	        /* Gouraud */
		initLight(1, 0xff);
		break;
	    case 3:
	        /* Textured /* eNX`t */
		initLight(0, 0xff);
		break;
	}
}

