/* $PSLibId: Runtime Library Release 3.6$ */
/*		    tuto5: cube with texture mapping
 *
 *         Copyright (C) 1993-1995 by Sony Computer Entertainment
 *			All rights Reserved
 *		Draw 3D objects (cube) with lighting texture
 *		  ̂eNX`̂̕`
 :
 */
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>

#define SCR_Z	(1024)		
#define	OTSIZE	(4096)

/* primitive buffer: v~eBu֘Ãobt@ */
typedef struct {
	DRAWENV		draw;		/* drawing environment: ` */
	DISPENV		disp;		/* display environment: \ */
	u_long		ot[OTSIZE];	/* OT: I[_Oe[u */
	POLY_FT4	s[6];		/* sides of cube: ̂̑ */
} DB;

/* light sourceiLocal Color Matrixj:
   Fi[JJ[}gbNXj */
static MATRIX	cmat = {
/* light source    #0, #1, #2, */
		ONE*3/4,  0,  0, /* R */
		ONE*3/4,  0,  0, /* G */
		ONE*3/4,  0,  0, /* B */
};

/* light vector (local light matrix)
   : xNgi[JCg}gbNXj */
static MATRIX lgtmat = {
	/*          X     Y     Z */
	          ONE,  ONE, ONE,	/*  #0 */
		    0,    0,    0,	/*      #1 */
		    0,    0,    0	/*      #2 */
};

static int pad_read(MATRIX *rottrans, MATRIX *rotlgt);
static void init_prim(DB *db, u_short tpage, u_short clut);

main()
{
	extern u_long	bgtex[];	/* trexture pattern (see bgtex.c) */
	
	DB	db[2];			/* double buffer */
	DB	*cdb	;		/* current buffer */
	MATRIX	rottrans;		/* rot-trans matrix */
	MATRIX		rotlgt;		/* light rotation matrix */
	MATRIX		light;		/* light matrix */
	CVECTOR		col[6];		/* cube color */
	
	int		i;		/* work */
	int		dmy, flg;	/* dummy */
	u_short		tpage, clut;	/* clut  and tpage */
	
	/* initialize environment for double buffer (interlace)
	 :_uobt@p̊ݒiC^[[X[hj
	 */
	init_system(320, 240, SCR_Z, 0);
	SetDefDrawEnv(&db[0].draw, 0, 0, 640, 480);
	SetDefDrawEnv(&db[1].draw, 0, 0, 640, 480);
	SetDefDispEnv(&db[0].disp, 0, 0, 640, 480);
	SetDefDispEnv(&db[1].disp, 0, 0, 640, 480);

	/* load texture and CLUT: eNX`ACLUT ̃[h */
	clut  = LoadClut(bgtex, 0, 480);
	tpage = LoadTPage(bgtex + 0x80, 0, 0, 640, 0, 256, 256);

	SetBackColor(64, 64, 64);	
	SetColorMatrix(&cmat);		

	/* set primitive parametes on buffer: v~eBuobt@̏ݒ */
	init_prim(&db[0], tpage, clut);	
	init_prim(&db[1], tpage, clut);	

	/* set surface colors. 
	 * When ShadeTex is enable, the unit value of the color is 0x80
	 * not 0xff.
	 : ̂̑ʂ̐F̐ݒ
	 * Pxl 0x80 ݒ肳ƂƂ̃eNX`FłB
	 */
	for (i = 0; i < 6; i++) {
		col[i].cd = db[0].s[0].code;	/* CODE */
		col[i].r  = 0x80;
		col[i].g  = 0x80;
		col[i].b  = 0x80;
	}

	SetDispMask(1);			/* start displaying: \Jn */
	PutDrawEnv(&db[0].draw);	/* set DRAWENV: `̐ݒ */
	PutDispEnv(&db[0].disp);	/* set DISPENV: \̐ݒ */

	while (pad_read(&rottrans, &rotlgt) == 0) {

		cdb = (cdb==db)? db+1: db;	/* change current buffer */
		ClearOTagR(cdb->ot, OTSIZE);	/* clear OT */

		/* Calcurate Matrix for the light source 
		   : }gbNX̐ݒ */
		MulMatrix0(&lgtmat, &rotlgt, &light);
		SetLightMatrix(&light);

		/* apend cubes into OT: ̂nsɓo^ */
		add_cubeFT4L(cdb->ot, cdb->s, &rottrans, col);
		
		VSync(0);
		ResetGraph(1);
		
		/* clear background: wĩNA */
		ClearImage(&cdb->draw.clip, 60, 120, 120);

		/* draw primitives: ` */
		/* DumpOTag(cdb->ot+OTSIZE-1);	/* for debug */
		DrawOTag(cdb->ot+OTSIZE-1);	
		FntFlush(-1);
	}
        /* close controller: Rg[̃N[Y */
	PadStop();			
	ResetGraph(1);
	StopCallback();
	return;
}

/* 
 * Analyzing PAD and Calcurating Matrix
 : Rg[̉͂ƁAϊ}gbNX̌vZ
 */
/* MATRIX *rottrans; 	rot-trans matrix: ̂̉]Esړ}gbNX */
/* MATRIX *rotlgt;	light source matrix: }gbNX */
static int pad_read(MATRIX *rottrans, MATRIX *rotlgt)
{
	/* angle of rotation for the cube: angle ̂̉]px */
	static SVECTOR	ang  = { 0, 0, 0};	
	
	/* ̉]px: angle of rotation for the light source */
	static SVECTOR	lgtang = {1024, -512, 1024};	
	
	/* translation vector: sړxNg */
	static VECTOR	vec  = {0, 0, SCR_Z};	

	/* read from controller: Rg[f[^ǂݍ */
	u_long	padd = PadRead(1);	

	int	ret = 0;
	
	/* quit program: vȌI */
	if (padd & PADselect) 	ret = -1;	

	/* change the rotation angles for the cube and the light source
	   : Ɨ̂̉]px̕ύX */
	if (padd & PADRup)	ang.vz += 32;
	if (padd & PADRdown)	ang.vz -= 32;
	if (padd & PADRleft) 	ang.vy += 32;
	if (padd & PADRright)	ang.vy -= 32;
	
	/* change the rotation angles only for the light source 
	   : ݂̂̉]px̕ύX */
	if (padd & PADLup)	lgtang.vx += 32;
	if (padd & PADLdown)	lgtang.vx -= 32;
	if (padd & PADLleft) 	lgtang.vy += 32;
	if (padd & PADLright)	lgtang.vy -= 32;

	/* distance from screen : _̋ */
	if (padd & PADL1)	vec.vz += 8;
	if (padd & PADR1) 	vec.vz -= 8;

	/* rotation matrix of the light source: ̉]}gNXvZ */
	RotMatrix(&lgtang, rotlgt);	
	MulMatrix(rotlgt, rottrans);

	/* set rot-trans-matrix of the cube: ̂̉]}gNXvZ */
	RotMatrix(&ang, rottrans);	
	TransMatrix(rottrans, &vec);	

	FntPrint("tuto5: lighting angle=(%d,%d,%d)\n",
		 lgtang.vx, lgtang.vy, lgtang.vz);
	return(ret);
}

/*
 *	Initialization assosiate with Primitives.
 :	v~eBu֘Ȁݒ
 */
/* DB	*db;	primitive buffer: v~eBuobt@ */
static void init_prim(DB *db, u_short tpage, u_short clut)
{
	int i;

	/* Textrued 4 point Polygon declared
	   : eNX`Sp`v~eBȕݒ */
	for(i = 0; i < 6; i++) {
		SetPolyFT4(&db->s[i]);	
		setUV4(&db->s[i], 0, 0, 0, 64, 64, 0, 64, 64);
		setRGB0(&db->s[i], 128, 128, 128);
		db->s[i].tpage = tpage;
		db->s[i].clut  = clut;
	}
}
