/* $PSLibId: Runtime Library Release 3.6$ */
/*		  tuto7: many cubes with local coordinates
 *
 *         Copyright (C) 1993-1995 by Sony Computer Entertainment
 *			All rights Reserved
 *
 *		      Drawing many 3D objects.
 :		     3D IuWFNg̕`
 *
 */
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>

#define SCR_Z	1024		
#define	OTSIZE	4096
#define NCUBE	256		/* max number of cubes:
				   \闧̂̐̍ől */

typedef struct {
	POLY_F4		s[6];		/* surface of cube: ̂̑ */
} CUBE;

typedef struct {
	DRAWENV		draw;		/* drawing environment: ` */
	DISPENV		disp;		/* display environment: \ */
	u_long		ot[OTSIZE];	/* OT: I[_Oe[u */
	CUBE		c[NCUBE];	/* pointer to CUBE: ̂̃f[^ */
} DB;

typedef struct {
	CVECTOR	col[6];			/* color of cube surface: ʂ̐F */
	SVECTOR	trans;			/* translation vector (local coord.)
					   : sړxNgi[JWj*/
} CUBEATTR;

/* Fi[JJ[}gbNXj */
static MATRIX	cmat = {
/*  light source   #0, #1, #2,
	 :     #0, #1, #2, */
		ONE*3/4,  0,  0, /* R */
		ONE*3/4,  0,  0, /* G */
		ONE*3/4,  0,  0, /* B */
};

/* the vector of the light source (Local light Matrixj 
   : xNgi[JCg}gbNXj */
static MATRIX lgtmat = {
	/*          X     Y     Z */
	          ONE,  ONE, ONE,	/*  #0 */
		    0,    0,    0,	/* #1 */
		    0,    0,    0	/* #2 */
};

static int pad_read(int *ncube, 
		MATRIX *world, MATRIX *local, MATRIX *rotlgt);
static void init_attr(CUBEATTR *attr, int nattr);
static init_prim(DB *db);

main()
{
	
	DB	db[2];		/* double buffer */
	DB	*cdb;		/* current buffer */
	
	/* attribute of cube: ̂̑ */
	CUBEATTR	attr[NCUBE];	
	
	/* world-screen matrix: [hXN[}gNX */
	MATRIX		ws;

	/* local-screen: [JXN[}gNX */
	MATRIX		ls;

	/* light source matrix: ̃[JXN[}gbNX */
	MATRIX		lls;		
	
	/* lighint matrix: ŏIIȌ}gbNX */
	MATRIX		light;

	/* number of cubes: \闧̂̐ */
	int		ncube = NCUBE/2;
	
	int		i;		/* work */
	long		dmy, flg;	/* dummy */

	/* set double buffer environment
	 :  _uobt@p̊ݒiC^[[X[hj*/
	init_system(320, 240, SCR_Z, 0);
	SetDefDrawEnv(&db[0].draw, 0, 0, 640, 480);
	SetDefDrawEnv(&db[1].draw, 0, 0, 640, 480);
	SetDefDispEnv(&db[0].disp, 0, 0, 640, 480);
	SetDefDispEnv(&db[1].disp, 0, 0, 640, 480);

	/* set background color: wiF̐ݒ */
	SetBackColor(64, 64, 64);	
	
	/* set local
	   color matrix: [JJ[}gbNX̐ݒ */
	SetColorMatrix(&cmat);		

	/* v~eBuobt@̏ݒ */
	init_prim(&db[0]);
	init_prim(&db[1]);
	init_attr(attr, NCUBE);

	/* display enable: \Jn */
	SetDispMask(1);		

	PutDrawEnv(&db[0].draw); /* set DRAWENV: `̐ݒ */
	PutDispEnv(&db[0].disp); /* set DISPENV: \̐ݒ */

	while (pad_read(&ncube, &ws, &ls, &lls) == 0) {

		cdb = (cdb==db)? db+1: db;	
		ClearOTagR(cdb->ot, OTSIZE);	

		/* Calcurate Matrix for the light source;
		 * Notice that MulMatrix() destroys current matrix.
		 : }gbNX̐ݒ
		 * Mulmatrix() ̓Jg}gNXj󂷂邱Ƃɒ
		 */
		PushMatrix();
		MulMatrix0(&lgtmat, &lls, &light);
		PopMatrix();
		SetLightMatrix(&light);
		
		/* put the primitives of cubes int OT: ̂̂nsւ̓o^ */
		limitRange(ncube, 1, NCUBE);
		
		/* add cubes to the OT
		 * note that only translation vector of the local
		 * screen matrix is changed   ncube ̗̂o^
		 : OT ɓo^
		 * [JXN[}gNẌړxNg
		 * XVĂ邱Ƃɒ
		 */ 
		for (i = 0; i < ncube; i++) {
			RotTrans(&attr[i].trans, (VECTOR *)ls.t, &flg);
			add_cubeF4L(cdb->ot, cdb->c[i].s, &ls, attr[i].col);
		}

		VSync(0);
		ResetGraph(1);

		/* clear background: wiNA */
		ClearImage(&cdb->draw.clip, 60, 120, 120);

		DrawOTag(cdb->ot+OTSIZE-1);	/* draw: ` */
		FntFlush(-1);			/* for debug */
	}
	/* close controller: Rg[̃N[Y */
	PadStop();			
	ResetGraph(1);
	StopCallback();
	return;
}


/* Analyzing PAD and Calcurating Matrix
 : Rg[̉͂ƁAϊ}gbNX̌vZ
 */
/* int	  *ncube	number of cubes: \闧̂̐ */
/* MATRIX *ws	 	rottrans matrix for all: Ŝ̉]}gbNX */
/* MATRIX *ls	 	rottrans matrix for each: ể]}gbNX */
/* MATRIX *lls		light local-screen matrix: ̉]}gbNX */
static int pad_read(int *ncube, 
		MATRIX *ws, MATRIX *ls, MATRIX *lls)
{
	/* PlayStation treats angle like as follows: 360= 4096
	   : Play Station ł́Apx 360= 4096 ň܂B*/
	
	/*; ]ƂƂ͎ȊO̐Etɉ]
	   ƂƓȂƂɒ */
	 
	/*  rotation angle of world: Ŝ̉]px */
	static SVECTOR	wang    = {0,  0,  0};	
	
	/* rotation angle for each cube: X̗̂̉]px */
	static SVECTOR	lang   = {0,  0,  0};	
	
	/* lotation angle for light source: ̉]px */
	static SVECTOR	lgtang = {1024, -512, 1024};	
	
	/* translation vector for all objects: Ŝ̕sړxNg */
	static VECTOR	vec    = {0,  0,  SCR_Z};

	/* scale of the each cube: ê̊gEk */
	static VECTOR	scale  = {1024, 1024, 1024, 0};
	
	SVECTOR	dwang, dlang;
	int	ret = 0;
	u_long	padd = PadRead(0);

	/* quit: I */
	if (padd & PADselect) 	ret = -1;	

	/* rotate all cubes : ́iŚj]B*/
	if (padd & PADRup)	wang.vz += 32;
	if (padd & PADRdown)	wang.vz -= 32;
	if (padd & PADRleft) 	wang.vy += 32;
	if (padd & PADRright)	wang.vy -= 32;

	/* rotate each cube: ݂̂] */
	if (padd & PADLup)	lang.vx += 32;
	if (padd & PADLdown)	lang.vx -= 32;
	if (padd & PADLleft) 	lang.vy += 32;
	if (padd & PADLright)	lang.vy -= 32;
	
	/* distance from screen : _̋ */
	if (padd & PADL1)	vec.vz += 8;
	if (padd & PADL2) 	vec.vz -= 8;

	/* change number of displayed cubes : \闧̂̐̕ύX */
	if (padd & PADR1)       (*ncube)++;
	if (padd & PADR2)	(*ncube)--;
	limitRange(*ncube, 1, NCUBE-1);

	FntPrint("objects = %d\n", *ncube);
	
	/* calcurate world-screen matrix
	  [hXN[}gbNX̌vZ */
	RotMatrix(&wang, ws);	
	TransMatrix(ws, &vec);

	/* calcurate matrix for each cubes.
	 * In this case, each local-screen matrix is the same because each
	 * cube rotates to the same direction.
	 : X̗̂̃[J}gbNX̌vZ 
	 * ̏ꍇ́Â͓ɉ]̂Ń[J}gNX͓
	 */
	RotMatrix(&lang, ls);
	
	/* make local-screen matrix:
	 * Notice the difference between  MulMatrix() and MulMatrix2()
	 : [JXN[}gbNX 
	 * MulMatrix()  MulMatrix2() ̈Ⴂɒ
	 */
	MulMatrix2(ws, ls); 
	
	/*: make light matrix:  ̉]}gbNX̌vZ */
	RotMatrix(&lgtang, lls);
	MulMatrix(lls, ls);

	/* scale of the local matrix represents the scale of the ls object.
	 : IuWFNg̃XP[̓[J}gNX̃XP[ŕ\ł */
	ScaleMatrix(ls, &scale);
	
	/*Setting Matrix for all the objectsj
	  : }gbNX̐ݒiŚj */
	SetRotMatrix(ws);	/* rotation: ]}gbNX */
	SetTransMatrix(ws);	/* translation: sړxNg */

	return(ret);
}

#define MIN_D 	64		/* minumus distance between each cube */
#define MAX_D	(SCR_Z/2)	/* maximum distance */
/* CUBEATTR	*attr,	attribute of cube: ̂̑ */
/* int		nattr	number of cube: ̂̐ */
static void init_attr(CUBEATTR *attr, int nattr)
{
	int	i;
	POLY_F4	templ;

	SetPolyF4(&templ);

	for (; nattr; nattr--, attr++) {
		for (i = 0; i < 6; i++) {
			attr->col[i].cd = templ.code;	/* sys code */
			attr->col[i].r  = rand();	/* R */
			attr->col[i].g  = rand();	/* G */
			attr->col[i].b  = rand();	/* B */
		}
		/* Set initial coordinates; X^[gʒu̐ݒ */
		attr->trans.vx = (rand()/MIN_D*MIN_D)%MAX_D-(MAX_D/2);
		attr->trans.vy = (rand()/MIN_D*MIN_D)%MAX_D-(MAX_D/2);
		attr->trans.vz = (rand()/MIN_D*MIN_D)%MAX_D-(MAX_D/2);
	}
}
	
/* DB	*db;	primivie buffer: v~eBuobt@ */
static init_prim(DB *db)
{
	int	i, j;

	for (i = 0; i < NCUBE; i++) 
		for (j = 0; j < 6; j++) 
			SetPolyF4(&db->c[i].s[j]);
}
