/* $PSLibId: Runtime Library Release 3.6$ */
/*			tuto9: cell type BG
 *
 *		 Version	Date		Design
 *		-----------------------------------------
 *		1.00		Jul,29,1994	suzu 
 *		2.00		May,22,1995	sachiko
 *
 *         Copyright (C) 1993-1995 by Sony Computer Entertainment
 * 			All rights Reserved
 *
 * Here is a sample program for coventional cell type BG estimation using
 * 8bit/4bit textures.
 * 8bit textured primitive is slower than 4bit textured primite.
 * this sample can draw 4-full-BGs in 1/60 sec using 8bit POLY_FT4 
 *
 * When you want to use special effect like magnify, rotate, and so 
 * on at the BG, you have to use POLY_FT4 instead of SPRT even though
 * POLY_FT4  is 2 times slower than SPRT.
 *
 * POLY_FT4 primitives are neccesary for magnified or rotated BG cells, 
 * but it is slower than SPRT. therefore if you do not need these functions,
 * you should use SPRT instead of POLY_FT4. Please define BG_SPRT when you
 * want to use SPRT primitevs as each BG cells.
 *
 :			Z^Cv BG ̎ 
 *
 *  	̃vÓA4bit/8bit eNX`gp BG ̑x]
 *	vOłB
 *	8bit eNX`̏ꍇ 4bit eNX`ɔׂđxቺ܂B
 *	̗ł POLY_FT4 ̃ACgpāA4 ̃tTCY BG `
 *	悵Ă܂B
 *
 *	BG_SPRT  define ƃXvCggp BG `悵܂B
 *
 *	BG ]Eg債ꍇ SPRT gp POLY_FT4
 *	gpKv܂BPOLY_FT4  SPRT `摬xቺ
 *	܂B]Ă̗̗lɓ{̃}bsÔ݂gpꍇ SPRT
 *	̕łB
 *
 *
 */
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>

/* BG Z̒Pʂ 32x32 */
#define BG_CELLX	32
#define BG_CELLY	32

#define BG_WIDTH	288	/* max BG width: BG ̍ő啝 */
#define BG_HEIGHT	192	/* max BG height: BG ̍ő卂 */

/* number of BG cells: Žij*/
#define BG_NX		(BG_WIDTH/BG_CELLX+2)
#define BG_NY		(BG_HEIGHT/BG_CELLY+2)

/* depth of OT: OT ̕\ 4 */
#define OTSIZE		4	

/* Define BG structure: BG \̂V` */
typedef struct {
	/* local DRAWENV for each BG: e BG Ƃ̕` */
	DRAWENV		env;	
	
	/* local OT for each BG: e BG Ƃ OT */
	u_long		ot[OTSIZE];	
	
	SPRT		cell[BG_NY*BG_NX];	/* BG cells (SPRT) */
	POLY_FT4	cell2[BG_NY*BG_NX];	/* BG cells (POLY_FT4) */
} BG;

/*
 * primitive double buffer: v~eBu_uobt@
 */
#define MAXBG	4			/* max 4 BGs: ő BG  */
typedef struct {
	DISPENV	disp;			/* display environment: \ */
	BG	bg[MAXBG];		/* BG */
} DB;

/* define this when you use SPRT BG
   : XvCg BGgpꍇi|ŜƂ́ARgAEgj */
#define BG_SPRT

/* define this when you use 4bit texture
   : 4bit eNX`gpꍇi8bit ̂Ƃ̓RgAEgj */
#define BG_4bit

static void bg_init(BG *bg, int x, int y);
static void bg_update(BG *bg, SVECTOR *mapxy);
static void bg_draw(BG *bg);
static int pad_read(SVECTOR *mapxy);

main()
{
	/* primitive double buffer: pPbg_uobt@̎ */
	DB	db[2];		
	
	/* current double buffer: ݒpPbgobt@փ|C^ */
	DB	*cdb;		
	
	/* current OT: ݒpPbgns */
	u_long	*ot;		
	
	/* current location of BG: bǧ݂̃}bvʒui͎gpȂj */
	SVECTOR	mapxy[4];	
	
	int	i;
	PadInit(0);		/* reset PAD: Rg[Zbg */
	ResetGraph(0);		/* reset GPU: OtBbNVXeZbg */

	/* clear frame buffer: t[_uobt@NA */
	{
		RECT rect;
		setRECT(&rect, 0, 0, 320, 480);
		ClearImage(&rect, 0, 0, 0);
	}

	/* define frame double buffer: _uobt@̒` */
	/*	Buffer0   #0:	(  0   0) -(320,240)
	 *	Buffer1   #1:	(  0,240) -(320,480)
	 */
	SetDefDispEnv(&db[0].disp,  0, 240, 320, 240);
	SetDefDispEnv(&db[1].disp,  0,   0, 320, 240);

	/* Set Initial value of primitive buffer
	 * link primitive list for BG here.
	 : v~eBuobt@̓ȅlݒ
	 * ŁABG p̃v~eBuXg̃N܂ŒĂ܂
	 */

	bg_init(&db[0].bg[0], 0, 4);		/* BG #0 */
	bg_init(&db[1].bg[0], 0, 4+240);

	bg_init(&db[0].bg[1], 20, 12);		/* BG #1 */
	bg_init(&db[1].bg[1], 20, 12+240);

	bg_init(&db[0].bg[2], 40, 20);		/* BG #2 */
	bg_init(&db[1].bg[2], 40, 20+240);

	bg_init(&db[0].bg[3], 60, 28);		/* BG #3 */
	bg_init(&db[1].bg[3], 60, 28+240);

	/* }bv̏lݒ */
	setVector(&mapxy[0],  0,  0,0);	/* default position (  0,  0) */
	setVector(&mapxy[1],256,256,0);	/* default position (256,256) */
	setVector(&mapxy[2],  0,256,0);	/* default position (  0,256) */
	setVector(&mapxy[3],256,  0,0);	/* default position (256,  0) */

	/* display enable: \Jn */
	SetDispMask(1);

	/* main loop: C[v */
	while (pad_read(mapxy) == 0) {		/* mapxy l */

		/* swapt primitive buffer ID
		   : ݒv~eBuobt@|C^lB*/
		cdb = (cdb==db)? db+1: db;

		/* update BG: BG XV */
		for (i = 0; i < 4; i++)
			bg_update(&cdb->bg[i], &mapxy[i]);

		/* swap double buffer: _uobt@Xbv */
		DrawSync(0);
		VSync(0);
		PutDispEnv(&cdb->disp); 

		/* draw 4 plane of BG: BG Sʕ`悷 */
		for (i = 0; i < 4; i++)
			bg_draw(&cdb->bg[i]);
	}

	/* quit: I */
        PadStop();		
	ResetGraph(1);
	StopCallback();
	return;
}

/***************************************************************************
 *
 *	BG management
 :	BG nhO
 *
 ***************************************************************************/
/* BG cell structure
 * This sample does not use clut member because it uses only 1 clut.
 * This sample does not use attr member because no attributes are defined here.
 * See bgmap.h for detail.
 : BG Z\
 * ̗ł CLUT ͂P{Ȃ̂ clut o͎gpȂB
 * ̗ł Ȃ̂ attr o͎gpȂB
 * bgmap.h Q 
 */
typedef struct {
	u_char	u, v;	/* cell texture UV: ZeNX`p^[W */
	u_short	clut;	/* cell texture CLUT: ZeNX`p^[ CLUT */
	u_long	attr;	/* attribute: Agr[g */
} CTYPE;

/* 2D array for cell type: Z^CvLqQz */
#include "bgmap.h"		

#ifdef BG_4bit
extern	u_long	bgtex[];	/* BG textureECLUT (4bit) */
#else
extern	u_long	bgtex8[];	/* BG textureECLUT (8bit) */
#endif

/*
 * initialize BG.
 * All parameters which would not be changed is set here.
 : BG ̏:炩߂ł邱Ƃ݂͂ȂŐݒ肵Ă܂B
 */
/* BG	*bg,	BG data: BG f[^ */
/* int	x,y	location on screen: XN[̕\ʒuiwj */
static void bg_init(BG *bg, int x, int y)
{
	SPRT		*cell;
	POLY_FT4	*cell2;
	u_short		clut, tpage;
	int		ix, iy;

	/* set drawing environment: `ݒ */
	SetDefDrawEnv(&bg->env, x, y, BG_WIDTH, BG_HEIGHT);

	/* load texture data and CLUT: eNX`EeNX` CLUT [h */

#ifdef BG_4bit	/* 4bit mode */
	tpage = LoadTPage(bgtex+0x80, 0, 0, 640, 0, 256, 256);
	clut  = LoadClut(bgtex, 0, 480);
#else		/* 8bit mode */
	tpage = LoadTPage(bgtex8+0x80, 1, 0, 640, 0, 256, 256);
	clut  = LoadClut(bgtex8, 0, 480);
#endif
	/* set default texture page: ftHgeNX`y[Wݒ */
	bg->env.tpage = tpage;

	/* clear local OT: [JnsNA */
	ClearOTag(bg->ot, OTSIZE);

	/* make primitive list for BG
	   : BG \v~eBuXg쐬B*/

	for (cell = bg->cell, cell2 = bg->cell2, iy = 0; iy < BG_NY; iy++)
	    for (ix = 0; ix < BG_NX; ix++, cell++, cell2++) {

#ifdef BG_SPRT
		SetSprt(cell);			/* SPRT Primitive */
		SetShadeTex(cell, 1);		/* ShadeTex forbidden */
		setWH(cell, BG_CELLX, BG_CELLY);/* Set the sizes of the cell */
		cell->clut = clut;		/* Set texture CLUT  */
		AddPrim(&bg->ot[0], cell);	/* Put SPRT primitive into OT */
#else
		SetPolyFT4(cell2);		/* POLY_FT4 Primitive */
		SetShadeTex(cell2, 1);		/* ShadeTex forbidden */
		cell2->tpage = tpage;		/* Set texture pages*/
		cell2->clut  = clut;		/* Set texture CLUT*/
		AddPrim(&bg->ot[0], cell2);	/* Put POLY_FT4 primitives 
						   into OT  */
#endif
	    }
}

/* updatea BG members: BG ̃oXV */
/* BG		*bg,	BG data: BG f[^ */
/* SVECTOR	*mapxy	map location: eNX` BG ւ̃}bvʒu */

static void bg_update(BG *bg, SVECTOR *mapxy)
{
	/* cell data for sprites: XvCgpZf[^ */
	SPRT		*cell;		
	
	/* cell data for polygon: |SpZf[^ */
	POLY_FT4	*cell2;		
	
	/* cell type: Z^Cv */
	CTYPE		*ctype;		
	
	/* absolute postion on map: }bṽItZbgl */
	int		mx, my;		
	
	/* relative position on map: }bv̒ł̃ItZbgl */
	int		dx, dy;		
	
	int		tx, ty;		/* work */
	int		x, y, ix, iy;

	/* (tx, ty) is rap-rounded at BG_CELLX*BG_MAPX 
	   : (tx, ty) ݈ʒuiBG_CELLX*BG_MAPX ŃbvEhj*/
	tx = (mapxy->vx)&(BG_CELLX*BG_MAPX-1);
	ty = (mapxy->vy)&(BG_CELLY*BG_MAPY-1);

	/*: tx  BG_CELLX Ŋl}bv̈ʒu (mx)
	 *  tx  BG_CELLX Ŋ]肪\ړ (dx)
	 */
	mx =  tx/BG_CELLX;	my =  ty/BG_CELLY;
	dx = -(tx%BG_CELLX);	dy = -(ty%BG_CELLY);

	/* update (u0,v0), (x0,y0) in the primitive list.
	   : v~eBuXg (u0,v0), (x0,y0) oύX */

	cell  = bg->cell;
	cell2 = bg->cell2;
	for (iy = y = 0; iy < BG_NY; iy++, y += BG_CELLY) {
		for (ix = x = 0; ix < BG_NX; ix++, x += BG_CELLX) {

			/* rap-rounded at BG_MAPX, BG_MAPY 
			   :(BG_MAPX, BG_MAPY) ŃbvEh */
			tx = (mx+ix)&(BG_MAPX-1);
			ty = (my+iy)&(BG_MAPY-1);

			/* get cell-type from the map data.
			 * Id code is stored  as a ASCII code in Map[][] 
			 : }bvZ^Cvl 
			 * Map[][] ̓LN^R[h ID Ă 
			 */ 
			ctype = &CType[(Map[ty])[tx]-'0'];
#ifdef BG_SPRT
			/* to reduce the memory access, update only
			 * (u0,v0), (x0, y0) menbers
			 : (u0,v0), (x0, y0) ݂̂XV
			 */
			setUV0(cell, ctype->u, ctype->v);
			setXY0(cell, x+dx, y+dy);
#else
			/* to reduce the memory access, update only
			 *  (u0,v0), (x0, y0) menbers (POLY_FT4)
			 : (u,v), (x, y) XV (POLY_FT4)
			 */
			setUVWH(cell2, ctype->u, ctype->v,
				BG_CELLX-1, BG_CELLY-1);
			setXYWH(cell2, x+dx, y+dy, BG_CELLX, BG_CELLY);
#endif
			cell++;
			cell2++;
		}
	}
}

/*
 * draw BG: BG `
 */
static void bg_draw(BG *bg)
{
	/* Update Drawing environment for each BG
	   : BG Ƃɐݒ肵`ɍXV */
	PutDrawEnv(&bg->env);	
	
	DrawOTag(bg->ot);	/* draw: ` */
}

/*
 * read controller: Rg[̃f[^ǂŁA BG ̈ړ߂B
 */
/* SVECTOR *mapxy	map location: eNX` BG ւ̃}bvʒu */
static int pad_read(SVECTOR	*mapxy)
{
	static SVECTOR	v[4] = {
		 2, 0, 0, 0,	0,  2, 0, 0,
		-2, 0, 0, 0,	0, -2, 0, 0,
	};

	int	i,id;
	u_long	padd = PadRead(1);

	/* quit: I */
	if(padd & PADselect) 	return(-1);

	id = 2;
	if (padd & (PADR1|PADL1|PADR2|PADL2)) id = 0;

	/* BG #0 i#2 if PADR1 or PADL1 is pushed)
	   : BG #0 i PADR1 ܂ PADL1 Ăꍇ BG #2 j*/
	if(padd & PADLup)	setVector(&v[id],  0, -2, 0);
	if(padd & PADLdown)	setVector(&v[id],  0,  2, 0);
	if(padd & PADLleft)	setVector(&v[id], -2,  0, 0);
	if(padd & PADLright)	setVector(&v[id],  2,  0, 0);

	/* BG #1 i#3 if PADR1 or PADL1 is pushed)
	   : BG #1 i PADR1 ܂ PADL1 Ăꍇ BG #3 j*/
	if(padd & PADRup)	setVector(&v[id+1],  0, -2, 0);
	if(padd & PADRdown)	setVector(&v[id+1],  0,  2, 0);
	if(padd & PADRleft)	setVector(&v[id+1], -2,  0, 0);
	if(padd & PADRright)	setVector(&v[id+1],  2,  0, 0);

	/* Add the motion range 
	   : ̈ړʂZ */
	for (i = 0; i < 4; i++)
		addVector(&mapxy[i], &v[i]);

	return(0);
}


